@@ -123,20 +123,20 @@ facet_grid <- function(rows = NULL, cols = NULL, scales = "fixed",
     cols <- NULL
   }
 
-  scales <- match.arg(scales, c("fixed", "free_x", "free_y", "free"))
+  scales <- arg_match0(scales %||% "fixed", c("fixed", "free_x", "free_y", "free"))
   free <- list(
     x = any(scales %in% c("free_x", "free")),
     y = any(scales %in% c("free_y", "free"))
   )
 
-  space <- match.arg(space, c("fixed", "free_x", "free_y", "free"))
+  space <- arg_match0(space %||% "fixed", c("fixed", "free_x", "free_y", "free"))
   space_free <- list(
     x = any(space %in% c("free_x", "free")),
     y = any(space %in% c("free_y", "free"))
   )
 
   if (!is.null(switch) && !switch %in% c("both", "x", "y")) {
-    abort("switch must be either 'both', 'x', or 'y'")
+    cli::cli_abort("{.arg switch} must be either {.val both}, {.val x}, or {.val y}")
   }
 
   facets_list <- grid_as_facets_list(rows, cols)
@@ -157,12 +157,21 @@ grid_as_facets_list <- function(rows, cols) {
   is_rows_vars <- is.null(rows) || is_quosures(rows)
   if (!is_rows_vars) {
     if (!is.null(cols)) {
-      abort("`rows` must be `NULL` or a `vars()` list if `cols` is a `vars()` list")
+      msg <- "{.arg rows} must be {.val NULL} or a {.fn vars} list if {.arg cols} is a {.fn vars} list"
+      # Native pipe have higher precedence than + so any type of gg object can be
+      # expected here, not just ggplot
+      if (inherits(rows, "gg")) {
+        msg <- c(
+          msg,
+          "i" = "Did you use {.code %>%} or {.code |>} instead of {.code +}?"
+        )
+      }
+      cli::cli_abort(msg)
     }
     # For backward-compatibility
     facets_list <- as_facets_list(rows)
     if (length(facets_list) > 2L) {
-      abort("A grid facet specification can't have more than two dimensions")
+      cli::cli_abort("A grid facet specification can't have more than two dimensions")
     }
     # Fill with empty quosures
     facets <- list(rows = quos(), cols = quos())
@@ -173,7 +182,7 @@ grid_as_facets_list <- function(rows, cols) {
 
   is_cols_vars <- is.null(cols) || is_quosures(cols)
   if (!is_cols_vars) {
-    abort("`cols` must be `NULL` or a `vars()` specification")
+    cli::cli_abort("{.arg cols} must be {.val NULL} or a {.fn vars} specification")
   }
 
   list(
@@ -189,16 +198,18 @@ grid_as_facets_list <- function(rows, cols) {
 FacetGrid <- ggproto("FacetGrid", Facet,
   shrink = TRUE,
 
-  compute_layout = function(data, params) {
+  compute_layout = function(self, data, params) {
     rows <- params$rows
     cols <- params$cols
 
+    check_facet_vars(names(rows), names(cols), name = snake_class(self))
+
     dups <- intersect(names(rows), names(cols))
     if (length(dups) > 0) {
-      abort(glue(
-        "Faceting variables can only appear in row or cols, not both.\n",
-        "Problems: ", paste0(dups, collapse = "'")
-      ))
+      cli::cli_abort(c(
+              "Faceting variables can only appear in {.arg rows} or {.arg cols}, not both.\n",
+        "i" = "Duplicated variables: {.val {dups}}"
+      ), call = call2(snake_class(self)))
     }
 
     base_rows <- combine_vars(data, params$plot_env, rows, drop = params$drop)
@@ -210,12 +221,18 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     base <- df.grid(base_rows, base_cols)
 
     if (nrow(base) == 0) {
-      return(new_data_frame(list(PANEL = factor(1L), ROW = 1L, COL = 1L, SCALE_X = 1L, SCALE_Y = 1L)))
+      return(data_frame0(
+        PANEL = factor(1L),
+        ROW = 1L,
+        COL = 1L,
+        SCALE_X = 1L,
+        SCALE_Y = 1L
+      ))
     }
 
     # Add margins
     base <- reshape_add_margins(base, list(names(rows), names(cols)), params$margins)
-    base <- unique(base)
+    base <- unique0(base)
 
     # Create panel info dataset
     panel <- id(base, drop = TRUE)
@@ -224,7 +241,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     rows <- if (!length(names(rows))) rep(1L, length(panel)) else id(base[names(rows)], drop = TRUE)
     cols <- if (!length(names(cols))) rep(1L, length(panel)) else id(base[names(cols)], drop = TRUE)
 
-    panels <- new_data_frame(c(list(PANEL = panel, ROW = rows, COL = cols), base))
+    panels <- data_frame0(PANEL = panel, ROW = rows, COL = cols, base)
     panels <- panels[order(panels$PANEL), , drop = FALSE]
     rownames(panels) <- NULL
 
@@ -235,7 +252,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
   },
   map_data = function(data, layout, params) {
     if (empty(data)) {
-      return(cbind(data, PANEL = integer(0)))
+      return(vec_cbind(data %|W|% NULL, PANEL = integer(0)))
     }
 
     rows <- params$rows
@@ -258,15 +275,16 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     # duplicating the data
     missing_facets <- setdiff(vars, names(facet_vals))
     if (length(missing_facets) > 0) {
-      to_add <- unique(layout[missing_facets])
+      to_add <- unique0(layout[missing_facets])
 
       data_rep <- rep.int(1:nrow(data), nrow(to_add))
       facet_rep <- rep(1:nrow(to_add), each = nrow(data))
 
       data <- unrowname(data[data_rep, , drop = FALSE])
-      facet_vals <- unrowname(cbind(
-        facet_vals[data_rep, ,  drop = FALSE],
-        to_add[facet_rep, , drop = FALSE]))
+      facet_vals <- unrowname(vec_cbind(
+        unrowname(facet_vals[data_rep, ,  drop = FALSE]),
+        unrowname(to_add[facet_rep, , drop = FALSE]))
+      )
     }
 
     # Add PANEL variable
@@ -276,6 +294,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     } else {
       facet_vals[] <- lapply(facet_vals[], as.factor)
       facet_vals[] <- lapply(facet_vals[], addNA, ifany = TRUE)
+      layout[] <- lapply(layout[], as.factor)
 
       keys <- join_keys(facet_vals, layout, by = vars)
 
@@ -285,15 +304,15 @@ FacetGrid <- ggproto("FacetGrid", Facet,
   },
   draw_panels = function(panels, layout, x_scales, y_scales, ranges, coord, data, theme, params) {
     if ((params$free$x || params$free$y) && !coord$is_free()) {
-      abort(glue("{snake_class(coord)} doesn't support free scales"))
+      cli::cli_abort("{.fn {snake_class(coord)}} doesn't support free scales")
     }
 
     cols <- which(layout$ROW == 1)
     rows <- which(layout$COL == 1)
     axes <- render_axes(ranges[cols], ranges[rows], coord, theme, transpose = TRUE)
 
-    col_vars <- unique(layout[names(params$cols)])
-    row_vars <- unique(layout[names(params$rows)])
+    col_vars <- unique0(layout[names(params$cols)])
+    row_vars <- unique0(layout[names(params$rows)])
     # Adding labels metadata, useful for labellers
     attr(col_vars, "type") <- "cols"
     attr(col_vars, "facet") <- "grid"
@@ -302,6 +321,9 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     strips <- render_strips(col_vars, row_vars, params$labeller, theme)
 
     aspect_ratio <- theme$aspect.ratio
+    if (!is.null(aspect_ratio) && (params$space_free$x || params$space_free$y)) {
+      cli::cli_abort("Free scales cannot be mixed with a fixed aspect ratio")
+    }
     if (is.null(aspect_ratio) && !params$free$x && !params$free$y) {
       aspect_ratio <- coord$aspect(ranges[[1]])
     }
@@ -333,7 +355,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
       heights <- vapply(ps, function(i) diff(ranges[[i]]$y.range), numeric(1))
       panel_heights <- unit(heights, "null")
     } else {
-      panel_heights <- rep(unit(1 * aspect_ratio, "null"), nrow)
+      panel_heights <- rep(unit(1 * abs(aspect_ratio), "null"), nrow)
     }
 
     panel_table <- gtable_matrix("layout", panel_table,
@@ -346,10 +368,14 @@ FacetGrid <- ggproto("FacetGrid", Facet,
       theme$panel.spacing.y %||% theme$panel.spacing)
 
     # Add axes
-    panel_table <- gtable_add_rows(panel_table, max_height(axes$x$top), 0)
-    panel_table <- gtable_add_rows(panel_table, max_height(axes$x$bottom), -1)
-    panel_table <- gtable_add_cols(panel_table, max_width(axes$y$left), 0)
-    panel_table <- gtable_add_cols(panel_table, max_width(axes$y$right), -1)
+    axis_height_top <- max_height(axes$x$top)
+    axis_height_bottom <- max_height(axes$x$bottom)
+    axis_width_left <- max_width(axes$y$left)
+    axis_width_right <- max_width(axes$y$right)
+    panel_table <- gtable_add_rows(panel_table, axis_height_top, 0)
+    panel_table <- gtable_add_rows(panel_table, axis_height_bottom, -1)
+    panel_table <- gtable_add_cols(panel_table, axis_width_left, 0)
+    panel_table <- gtable_add_cols(panel_table, axis_width_right, -1)
     panel_pos_col <- panel_cols(panel_table)
     panel_pos_rows <- panel_rows(panel_table)
 
@@ -367,7 +393,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     panel_pos_col <- panel_cols(panel_table)
     if (switch_x) {
       if (!is.null(strips$x$bottom)) {
-        if (inside_x) {
+        if (inside_x || as.numeric(axis_height_bottom) == 0) {
           panel_table <- gtable_add_rows(panel_table, max_height(strips$x$bottom), -2)
           panel_table <- gtable_add_grob(panel_table, strips$x$bottom, -2, panel_pos_col$l, clip = "on", name = paste0("strip-b-", seq_along(strips$x$bottom)), z = 2)
         } else {
@@ -378,7 +404,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
       }
     } else {
       if (!is.null(strips$x$top)) {
-        if (inside_x) {
+        if (inside_x || as.numeric(axis_height_top) == 0) {
           panel_table <- gtable_add_rows(panel_table, max_height(strips$x$top), 1)
           panel_table <- gtable_add_grob(panel_table, strips$x$top, 2, panel_pos_col$l, clip = "on", name = paste0("strip-t-", seq_along(strips$x$top)), z = 2)
         } else {
@@ -391,7 +417,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
     panel_pos_rows <- panel_rows(panel_table)
     if (switch_y) {
       if (!is.null(strips$y$left)) {
-        if (inside_y) {
+        if (inside_y || as.numeric(axis_width_left) == 0) {
           panel_table <- gtable_add_cols(panel_table, max_width(strips$y$left), 1)
           panel_table <- gtable_add_grob(panel_table, strips$y$left, panel_pos_rows$t, 2, clip = "on", name = paste0("strip-l-", seq_along(strips$y$left)), z = 2)
         } else {
@@ -402,7 +428,7 @@ FacetGrid <- ggproto("FacetGrid", Facet,
       }
     } else {
       if (!is.null(strips$y$right)) {
-        if (inside_y) {
+        if (inside_y || as.numeric(axis_width_right) == 0) {
           panel_table <- gtable_add_cols(panel_table, max_width(strips$y$right), -2)
           panel_table <- gtable_add_grob(panel_table, strips$y$right, panel_pos_rows$t, -2, clip = "on", name = paste0("strip-r-", seq_along(strips$y$right)), z = 2)
         } else {
@@ -426,6 +452,6 @@ ulevels <- function(x) {
     x <- addNA(x, TRUE)
     factor(levels(x), levels(x), exclude = NULL)
   } else {
-    sort(unique(x))
+    sort(unique0(x))
   }
 }
