@@ -79,9 +79,10 @@ NULL
 #' }
 facet_wrap <- function(facets, nrow = NULL, ncol = NULL, scales = "fixed",
                        shrink = TRUE, labeller = "label_value", as.table = TRUE,
-                       switch = NULL, drop = TRUE, dir = "h", strip.position = 'top') {
-  scales <- match.arg(scales, c("fixed", "free_x", "free_y", "free"))
-  dir <- match.arg(dir, c("h", "v"))
+                       switch = deprecated(), drop = TRUE, dir = "h",
+                       strip.position = 'top') {
+  scales <- arg_match0(scales %||% "fixed", c("fixed", "free_x", "free_y", "free"))
+  dir <- arg_match0(dir, c("h", "v"))
   free <- list(
     x = any(scales %in% c("free_x", "free")),
     y = any(scales %in% c("free_y", "free"))
@@ -93,20 +94,22 @@ facet_wrap <- function(facets, nrow = NULL, ncol = NULL, scales = "fixed",
   # Flatten all facets dimensions into a single one
   facets <- wrap_as_facets_list(facets)
 
-  if (!is.null(switch)) {
-    .Deprecated("strip.position", old = "switch")
+  if (lifecycle::is_present(switch) && !is.null(switch)) {
+    deprecate_warn0("2.2.0", "facet_wrap(switch)", "facet_wrap(strip.position)")
     strip.position <- if (switch == "x") "bottom" else "left"
   }
-  strip.position <- match.arg(strip.position, c("top", "bottom", "left", "right"))
+  strip.position <- arg_match0(strip.position, c("top", "bottom", "left", "right"))
+  if (!(is.null(ncol) || (is_integerish(ncol, 1) && ncol > 0))) {
+    cli::cli_abort("{.arg ncol} must be a positive scalar integer or {.val NULL}")
+  }
+  if (!(is.null(nrow) || (is_integerish(nrow, 1) && nrow > 0))) {
+    cli::cli_abort("{.arg nrow} must be a positive scalar integer or {.val NULL}")
+  }
   if (identical(dir, "v")) {
     # swap
-    nrow_swap <- ncol
-    ncol_swap <- nrow
-    nrow <- sanitise_dim(nrow_swap)
-    ncol <- sanitise_dim(ncol_swap)
-  } else {
-    nrow <- sanitise_dim(nrow)
-    ncol <- sanitise_dim(ncol)
+    tmp <- ncol
+    ncol <- nrow
+    nrow <- tmp
   }
 
   ggproto(NULL, FacetWrap,
@@ -138,35 +141,37 @@ wrap_as_facets_list <- function(x) {
 FacetWrap <- ggproto("FacetWrap", Facet,
   shrink = TRUE,
 
-  compute_layout = function(data, params) {
+  compute_layout = function(self, data, params) {
     vars <- params$facets
     if (length(vars) == 0) {
       return(layout_null())
     }
 
-    base <- unrowname(
-      combine_vars(data, params$plot_env, vars, drop = params$drop)
-    )
+    check_facet_vars(names(vars), name = snake_class(self))
+
+    base <- combine_vars(data, params$plot_env, vars, drop = params$drop)
 
     id <- id(base, drop = TRUE)
     n <- attr(id, "n")
 
     dims <- wrap_dims(n, params$nrow, params$ncol)
-    layout <- new_data_frame(list(PANEL = factor(id, levels = seq_len(n))))
-
-    if (params$as.table) {
-      layout$ROW <- as.integer((id - 1L) %/% dims[2] + 1L)
-    } else {
-      layout$ROW <- as.integer(dims[1] - (id - 1L) %/% dims[2])
-    }
-    layout$COL <- as.integer((id - 1L) %% dims[2] + 1L)
+    layout <- data_frame0(
+      PANEL = factor(id, levels = seq_len(n)),
+      ROW = if (params$as.table) {
+        as.integer((id - 1L) %/% dims[2] + 1L)
+      } else {
+        as.integer(dims[1] - (id - 1L) %/% dims[2])
+      },
+      COL = as.integer((id - 1L) %% dims[2] + 1L),
+      .size = length(id)
+    )
 
     # For vertical direction, flip row and col
     if (identical(params$dir, "v")) {
       layout[c("ROW", "COL")] <- layout[c("COL", "ROW")]
     }
 
-    panels <- cbind(layout, unrowname(base))
+    panels <- vec_cbind(layout, base)
     panels <- panels[order(panels$PANEL), , drop = FALSE]
     rownames(panels) <- NULL
 
@@ -178,7 +183,7 @@ FacetWrap <- ggproto("FacetWrap", Facet,
   },
   map_data = function(data, layout, params) {
     if (empty(data)) {
-      return(cbind(data, PANEL = integer(0)))
+      return(vec_cbind(data %|W|% NULL, PANEL = integer(0)))
     }
 
     vars <- params$facets
@@ -190,19 +195,21 @@ FacetWrap <- ggproto("FacetWrap", Facet,
 
     facet_vals <- eval_facets(vars, data, params$.possible_columns)
     facet_vals[] <- lapply(facet_vals[], as.factor)
+    layout[] <- lapply(layout[], as.factor)
 
     missing_facets <- setdiff(names(vars), names(facet_vals))
     if (length(missing_facets) > 0) {
 
-      to_add <- unique(layout[missing_facets])
+      to_add <- unique0(layout[missing_facets])
 
       data_rep <- rep.int(1:nrow(data), nrow(to_add))
       facet_rep <- rep(1:nrow(to_add), each = nrow(data))
 
-      data <- unrowname(data[data_rep, , drop = FALSE])
-      facet_vals <- unrowname(cbind(
+      data <- data[data_rep, , drop = FALSE]
+      facet_vals <- vec_cbind(
         facet_vals[data_rep, ,  drop = FALSE],
-        to_add[facet_rep, , drop = FALSE]))
+        to_add[facet_rep, , drop = FALSE]
+      )
     }
 
     keys <- join_keys(facet_vals, layout, by = names(vars))
@@ -210,9 +217,9 @@ FacetWrap <- ggproto("FacetWrap", Facet,
     data$PANEL <- layout$PANEL[match(keys$x, keys$y)]
     data
   },
-  draw_panels = function(panels, layout, x_scales, y_scales, ranges, coord, data, theme, params) {
+  draw_panels = function(self, panels, layout, x_scales, y_scales, ranges, coord, data, theme, params) {
     if ((params$free$x || params$free$y) && !coord$is_free()) {
-      abort(glue("{snake_class(coord)} doesn't support free scales"))
+      cli::cli_abort("{.fn {snake_class(self)}} can't use free scales with {.fn {snake_class(coord)}}")
     }
 
     if (inherits(coord, "CoordFlip")) {
@@ -240,7 +247,7 @@ FacetWrap <- ggproto("FacetWrap", Facet,
 
     if (length(params$facets) == 0) {
       # Add a dummy label
-      labels_df <- new_data_frame(list("(all)" = "(all)"), n = 1)
+      labels_df <- data_frame0("(all)" = "(all)", .size = 1)
     } else {
       labels_df <- layout[names(params$facets)]
     }
@@ -313,30 +320,84 @@ FacetWrap <- ggproto("FacetWrap", Facet,
     )
     # Add back missing axes
     if (any(empties)) {
-      first_row <- which(apply(empties, 1, any))[1] - 1
-      first_col <- which(apply(empties, 2, any))[1] - 1
-      row_panels <- which(layout$ROW == first_row & layout$COL > first_col)
-      row_pos <- convertInd(layout$ROW[row_panels], layout$COL[row_panels], nrow)
-      row_axes <- axes$x$bottom[layout$SCALE_X[row_panels]]
-      col_panels <- which(layout$ROW > first_row & layout$COL == first_col)
-      col_pos <- convertInd(layout$ROW[col_panels], layout$COL[col_panels], nrow)
-      col_axes <- axes$y$right[layout$SCALE_Y[col_panels]]
+      row_ind <- row(empties)
+      col_ind <- col(empties)
       inside <- (theme$strip.placement %||% "inside") == "inside"
-      if (params$strip.position == "bottom" &&
-          !inside &&
-          any(!vapply(row_axes, is.zero, logical(1))) &&
-          !params$free$x) {
-        warn("Suppressing axis rendering when strip.position = 'bottom' and strip.placement == 'outside'")
-      } else {
-        axis_mat_x_bottom[row_pos] <- row_axes
+      empty_bottom <- apply(empties, 2, function(x) c(diff(x) == 1, FALSE))
+      if (any(empty_bottom)) {
+        pos <- which(empty_bottom)
+        panel_loc <- data_frame0(
+          ROW = row_ind[pos],
+          COL = col_ind[pos],
+          .size = length(pos)
+        )
+        panels <- vec_match(panel_loc, layout[, c("ROW", "COL")])
+        x_axes <- axes$x$bottom[layout$SCALE_X[panels]]
+        if (params$strip.position == "bottom" &&
+            !inside &&
+            any(!vapply(x_axes, is.zero, logical(1))) &&
+            !params$free$x) {
+          cli::cli_warn("Suppressing axis rendering when {.code strip.position = \"bottom\"} and {.code strip.placement == \"outside\"}")
+        } else {
+          axis_mat_x_bottom[pos] <- x_axes
+        }
       }
-      if (params$strip.position == "right" &&
-          !inside &&
-          any(!vapply(col_axes, is.zero, logical(1))) &&
-          !params$free$y) {
-        warn("Suppressing axis rendering when strip.position = 'right' and strip.placement == 'outside'")
-      } else {
-        axis_mat_y_right[col_pos] <- col_axes
+      empty_top <- apply(empties, 2, function(x) c(FALSE, diff(x) == -1))
+      if (any(empty_top)) {
+        pos <- which(empty_top)
+        panel_loc <- data_frame0(
+          ROW = row_ind[pos],
+          COL = col_ind[pos],
+          .size = length(pos)
+        )
+        panels <- vec_match(panel_loc, layout[, c("ROW", "COL")])
+        x_axes <- axes$x$top[layout$SCALE_X[panels]]
+        if (params$strip.position == "top" &&
+            !inside &&
+            any(!vapply(x_axes, is.zero, logical(1))) &&
+            !params$free$x) {
+          cli::cli_warn("Suppressing axis rendering when {.code strip.position = \"top\"} and {.code strip.placement == \"outside\"}")
+        } else {
+          axis_mat_x_top[pos] <- x_axes
+        }
+      }
+      empty_right <- t(apply(empties, 1, function(x) c(diff(x) == 1, FALSE)))
+      if (any(empty_right)) {
+        pos <- which(empty_right)
+        panel_loc <- data_frame0(
+          ROW = row_ind[pos],
+          COL = col_ind[pos],
+          .size = length(pos)
+        )
+        panels <- vec_match(panel_loc, layout[, c("ROW", "COL")])
+        y_axes <- axes$y$right[layout$SCALE_Y[panels]]
+        if (params$strip.position == "right" &&
+            !inside &&
+            any(!vapply(y_axes, is.zero, logical(1))) &&
+            !params$free$y) {
+          cli::cli_warn("Suppressing axis rendering when {.code strip.position = \"right\"} and {.code strip.placement == \"outside\"}")
+        } else {
+          axis_mat_y_right[pos] <- y_axes
+        }
+      }
+      empty_left <- t(apply(empties, 1, function(x) c(FALSE, diff(x) == -1)))
+      if (any(empty_left)) {
+        pos <- which(empty_left)
+        panel_loc <- data_frame0(
+          ROW = row_ind[pos],
+          COL = col_ind[pos],
+          .size = length(pos)
+        )
+        panels <- vec_match(panel_loc, layout[, c("ROW", "COL")])
+        y_axes <- axes$y$left[layout$SCALE_Y[panels]]
+        if (params$strip.position == "left" &&
+            !inside &&
+            any(!vapply(y_axes, is.zero, logical(1))) &&
+            !params$free$y) {
+          cli::cli_warn("Suppressing axis rendering when {.code strip.position = \"left\"} and {.code strip.placement == \"outside\"}")
+        } else {
+          axis_mat_y_left[pos] <- y_axes
+        }
       }
     }
     panel_table <- weave_tables_row(panel_table, axis_mat_x_top, -1, axis_height_top, "axis-t", 3)
@@ -390,58 +451,6 @@ FacetWrap <- ggproto("FacetWrap", Facet,
 
 # Helpers -----------------------------------------------------------------
 
-#' Sanitise the number of rows or columns
-#'
-#' Cleans up the input to be an integer greater than or equal to one, or
-#' `NULL`. Intended to be used on the `nrow` and `ncol`
-#' arguments of `facet_wrap()`.
-#' @param n Hopefully an integer greater than or equal to one, or `NULL`,
-#' though other inputs are handled.
-#' @return An integer greater than or equal to one, or `NULL`.
-#' @note If the length of the input is greater than one, only the first element
-#' is returned, with a warning.
-#' If the input is not an integer, it will be coerced to be one.
-#' If the value is less than one, `NULL` is returned, effectively ignoring
-#' the argument.
-#' Multiple warnings may be generated.
-#' @examples
-#' # Valid input just gets returns unchanged
-#' sanitise_dim(1)
-#' sanitise_dim(NULL)
-#'
-#' # Only the first element of vectors get returned
-#' sanitise_dim(10:1)
-#' # Non-integer values are coerced to integer
-#' sanitise_dim(pi)
-#' # Missing values, values less than one and non-numeric values are
-#' # treated as NULL
-#' sanitise_dim(NA_integer_)
-#' sanitise_dim(0)
-#' sanitise_dim("foo")
-#' @noRd
-sanitise_dim <- function(n) {
-  xname <- paste0("`", deparse(substitute(n)), "`")
-  if (length(n) == 0) {
-    if (!is.null(n)) {
-      warn(glue("{xname} has length zero and will be treated as NULL."))
-    }
-    return(NULL)
-  }
-  if (length(n) > 1) {
-    warn(glue("Only the first value of {xname} will be used."))
-    n <- n[1]
-  }
-  if (!is.numeric(n) || (!is.na(n) && n != round(n))) {
-    warn(glue("Coercing {xname} to be an integer."))
-    n <- as.integer(n)
-  }
-  if (is.na(n) || n < 1) {
-    warn(glue("{xname} is missing or less than 1 and will be treated as NULL."))
-    return(NULL)
-  }
-  n
-}
-
 #' Arrange 1d structure into a grid
 #'
 #' @param n length of structure
@@ -462,7 +471,10 @@ wrap_dims <- function(n, nrow = NULL, ncol = NULL) {
     nrow <- ceiling(n / ncol)
   }
   if (nrow * ncol < n) {
-    abort("The given dimensions cannot hold all panels. Please increase `ncol` or `nrow`")
+    cli::cli_abort(c(
+      "Need {n} panels, but together {.arg nrow} and {.arg ncol} only provide {nrow * ncol}",
+      i = "Please increase {.arg ncol} and/or {.arg nrow}"
+    ))
   }
 
   c(nrow, ncol)
