@@ -40,7 +40,13 @@
 #' automatically determines the orientation from the aesthetic mapping. In the
 #' rare event that this fails it can be given explicitly by setting `orientation`
 #' to either `"x"` or `"y"`. See the *Orientation* section for more detail.
-#' @param width Bar width. By default, set to 90% of the resolution of the data.
+#' @param just Adjustment for column placement. Set to `0.5` by default, meaning
+#'   that columns will be centered about axis breaks. Set to `0` or `1` to place
+#'   columns to the left/right of axis breaks. Note that this argument may have
+#'   unintended behaviour when used with alternative positions, e.g.
+#'   `position_dodge()`.
+#' @param width Bar width. By default, set to 90% of the [resolution()] of the
+#'   data.
 #' @param geom,stat Override the default connection between `geom_bar()` and
 #'   `stat_count()`.
 #' @examples
@@ -80,9 +86,17 @@
 #' ggplot(df, aes(x)) + geom_bar()
 #' # cf. a histogram of the same data
 #' ggplot(df, aes(x)) + geom_histogram(binwidth = 0.5)
+#'
+#' # Use `just` to control how columns are aligned with axis breaks:
+#' df <- data.frame(x = as.Date(c("2020-01-01", "2020-02-01")), y = 1:2)
+#' # Columns centered on the first day of the month
+#' ggplot(df, aes(x, y)) + geom_col(just = 0.5)
+#' # Columns begin on the first day of the month
+#' ggplot(df, aes(x, y)) + geom_col(just = 1)
 geom_bar <- function(mapping = NULL, data = NULL,
                      stat = "count", position = "stack",
                      ...,
+                     just = 0.5,
                      width = NULL,
                      na.rm = FALSE,
                      orientation = NA,
@@ -96,7 +110,8 @@ geom_bar <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
+      just = just,
       width = width,
       na.rm = na.rm,
       orientation = orientation,
@@ -123,22 +138,32 @@ GeomBar <- ggproto("GeomBar", GeomRect,
     params
   },
 
-  extra_params = c("na.rm", "orientation"),
+  extra_params = c("just", "na.rm", "orientation"),
 
   setup_data = function(data, params) {
     data$flipped_aes <- params$flipped_aes
     data <- flip_data(data, params$flipped_aes)
     data$width <- data$width %||%
       params$width %||% (resolution(data$x, FALSE) * 0.9)
+    data$just <- params$just %||% 0.5
     data <- transform(data,
       ymin = pmin(y, 0), ymax = pmax(y, 0),
-      xmin = x - width / 2, xmax = x + width / 2, width = NULL
+      xmin = x - width * just, xmax = x + width * (1 - just),
+      width = NULL, just = NULL
     )
     flip_data(data, params$flipped_aes)
   },
 
-  draw_panel = function(self, data, panel_params, coord, width = NULL, flipped_aes = FALSE) {
+  draw_panel = function(self, data, panel_params, coord, lineend = "butt",
+                        linejoin = "mitre", width = NULL, flipped_aes = FALSE) {
     # Hack to ensure that width is detected as a parameter
-    ggproto_parent(GeomRect, self)$draw_panel(data, panel_params, coord)
-  }
+    ggproto_parent(GeomRect, self)$draw_panel(
+      data,
+      panel_params,
+      coord,
+      lineend = lineend,
+      linejoin = linejoin
+    )
+  },
+  rename_size = TRUE
 )
