@@ -43,7 +43,10 @@ geom_jitter <- function(mapping = NULL, data = NULL,
                         inherit.aes = TRUE) {
   if (!missing(width) || !missing(height)) {
     if (!missing(position)) {
-      abort("You must specify either `position` or `width`/`height`.")
+      cli::cli_abort(c(
+        "both {.arg position} and {.arg width}/{.arg height} are supplied",
+        "i" = "Only use one approach to alter the position"
+      ))
     }
 
     position <- position_jitter(width = width, height = height)
@@ -57,7 +60,7 @@ geom_jitter <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       na.rm = na.rm,
       ...
     )
