@@ -50,12 +50,13 @@
 #' # Which seems like a lot of work, but then it's easy to add on
 #' # other features in this coordinate system, e.g.:
 #'
+#' set.seed(1)
 #' stream <- data.frame(
 #'   x = cumsum(runif(50, max = 0.1)),
 #'   y = cumsum(runif(50,max = 0.1))
 #' )
 #'
-#' p + geom_line(data = stream, colour = "grey30", size = 5)
+#' p + geom_line(data = stream, colour = "grey30", linewidth = 5)
 #'
 #' # And if the positions are in longitude and latitude, you can use
 #' # coord_map to produce different map projections.
@@ -93,7 +94,7 @@ geom_polygon <- function(mapping = NULL, data = NULL,
     position = position,
     show.legend = show.legend,
     inherit.aes = inherit.aes,
-    params = list(
+    params = list2(
       na.rm = na.rm,
       rule = rule,
       ...
@@ -106,7 +107,9 @@ geom_polygon <- function(mapping = NULL, data = NULL,
 #' @usage NULL
 #' @export
 GeomPolygon <- ggproto("GeomPolygon", Geom,
-  draw_panel = function(data, panel_params, coord, rule = "evenodd") {
+  draw_panel = function(self, data, panel_params, coord, rule = "evenodd",
+                        lineend = "butt", linejoin = "round", linemitre = 10) {
+    data <- check_linewidth(data, snake_class(self))
     n <- nrow(data)
     if (n == 1) return(zeroGrob())
 
@@ -130,18 +133,21 @@ GeomPolygon <- ggproto("GeomPolygon", Geom,
           gp = gpar(
             col = first_rows$colour,
             fill = alpha(first_rows$fill, first_rows$alpha),
-            lwd = first_rows$size * .pt,
-            lty = first_rows$linetype
+            lwd = first_rows$linewidth * .pt,
+            lty = first_rows$linetype,
+            lineend = lineend,
+            linejoin = linejoin,
+            linemitre = linemitre
           )
         )
       )
     } else {
       if (utils::packageVersion('grid') < "3.6") {
-        abort("Polygons with holes requires R 3.6 or above")
+        cli::cli_abort("Polygons with holes requires R 3.6 or above")
       }
       # Sort by group to make sure that colors, fill, etc. come in same order
       munched <- munched[order(munched$group, munched$subgroup), ]
-      id <- match(munched$subgroup, unique(munched$subgroup))
+      id <- match(munched$subgroup, unique0(munched$subgroup))
 
       # For gpar(), there is one entry per polygon (not one entry per point).
       # We'll pull the first value from each group, and assume all these values
@@ -158,16 +164,18 @@ GeomPolygon <- ggproto("GeomPolygon", Geom,
           gp = gpar(
             col = first_rows$colour,
             fill = alpha(first_rows$fill, first_rows$alpha),
-            lwd = first_rows$size * .pt,
-            lty = first_rows$linetype
+            lwd = first_rows$linewidth * .pt,
+            lty = first_rows$linetype,
+            lineend = lineend,
+            linejoin = linejoin,
+            linemitre = linemitre
           )
         )
       )
     }
-
   },
 
-  default_aes = aes(colour = "NA", fill = "grey20", size = 0.5, linetype = 1,
+  default_aes = aes(colour = NA, fill = "grey20", linewidth = 0.5, linetype = 1,
     alpha = NA, subgroup = NULL),
 
   handle_na = function(data, params) {
@@ -176,6 +184,20 @@ GeomPolygon <- ggproto("GeomPolygon", Geom,
 
   required_aes = c("x", "y"),
 
-  draw_key = draw_key_polygon
+  draw_key = draw_key_polygon,
+
+  rename_size = TRUE
 )
 
+# Assigning pathGrob in .onLoad ensures that packages that subclass GeomPolygon
+# do not install with error `possible error in 'pathGrob(munched$x, munched$y, ':
+# unused argument (pathId = munched$group)` despite the fact that this is correct
+# usage
+pathGrob <- NULL
+on_load(
+  if (getRversion() < as.numeric_version("3.6")) {
+    pathGrob <- function(..., pathId.lengths) {
+      grid::pathGrob(...)
+    }
+  }
+)
