@@ -72,7 +72,7 @@ guides <- function(...) {
 
   idx_false <- vapply(args, isFALSE, FUN.VALUE = logical(1L))
   if (isTRUE(any(idx_false))) {
-    warn('`guides(<scale> = FALSE)` is deprecated. Please use `guides(<scale> = "none")` instead.')
+    deprecate_warn0("3.3.4", "guides(`<scale>` = 'cannot be `FALSE`. Use \"none\" instead')")
     args[idx_false] <- "none"
   }
 
@@ -181,7 +181,7 @@ validate_guide <- function(guide) {
   else if (inherits(guide, "guide"))
     guide
   else
-    abort(glue("Unknown guide: {guide}"))
+    cli::cli_abort("Unknown guide: {guide}")
 }
 
 # train each scale in scales and generate the definition of guide
@@ -200,7 +200,7 @@ guides_train <- function(scales, theme, guides, labels) {
       if (identical(guide, "none") || inherits(guide, "guide_none")) next
 
       if (isFALSE(guide)) {
-        warn('It is deprecated to specify `guide = FALSE` to remove a guide. Please use `guide = "none"` instead.')
+        deprecate_warn0("3.3.4", I("The `guide` argument in `scale_*()` cannot be `FALSE`. This "), I('"none"'))
         next
       }
 
@@ -210,7 +210,7 @@ guides_train <- function(scales, theme, guides, labels) {
 
       # check the consistency of the guide and scale.
       if (!identical(guide$available_aes, "any") && !any(scale$aesthetics %in% guide$available_aes)) {
-        abort(glue("Guide '{guide$name}' cannot be used for '{scale$aesthetics}'."))
+        cli::cli_abort("Guide {.var {guide$name}} cannot be used for {.field {scale$aesthetics}}.")
       }
 
       guide$title <- scale$make_title(guide$title %|W|% scale$name %|W|% labels[[output]])
@@ -256,7 +256,10 @@ guides_gengrob <- function(gdefs, theme) {
     function(g) {
       g$title.position <- g$title.position %||% switch(g$direction, vertical = "top", horizontal = "left")
       if (!g$title.position %in% c("top", "bottom", "left", "right")) {
-        abort(glue("title position '{g$title.position}' is invalid"))
+        cli::cli_abort(c(
+          "Title position {.val {g$title.position}} is invalid",
+          "i" = "Use one of {.val top}, {.val bottom}, {.val left}, or {.val right}"
+        ))
       }
       g
     })
@@ -270,8 +273,10 @@ guides_build <- function(ggrobs, theme) {
   theme$legend.spacing.y <- theme$legend.spacing.y  %||% theme$legend.spacing
   theme$legend.spacing.x <- theme$legend.spacing.x  %||% theme$legend.spacing
 
-  widths <- do.call("unit.c", lapply(ggrobs, function(g)sum(g$widths)))
-  heights <- do.call("unit.c", lapply(ggrobs, function(g)sum(g$heights)))
+  widths <- lapply(ggrobs, function(g) sum(g$widths))
+  widths <- inject(unit.c(!!!widths))
+  heights <- lapply(ggrobs, function(g) sum(g$heights))
+  heights <- inject(unit.c(!!!heights))
 
   # Set the justification of each legend within the legend box
   # First value is xjust, second value is yjust
@@ -358,11 +363,9 @@ guide_transform <- function(guide, coord, panel_params) UseMethod("guide_transfo
 
 #' @export
 guide_transform.default <- function(guide, coord, panel_params) {
-  abort(glue(
-    "Guide with class ",
-    glue_collapse(class(guide), " / "),
-    " does not implement guide_transform(). ",
-    "Did you mean to use guide_axis()?"
+  cli::cli_abort(c(
+    "Guide with class {.cls {class(guide)}} does not implement {.fn guide_transform}",
+    "i" = "Did you mean to use {.fn guide_axis}?"
   ))
 }
 
@@ -376,6 +379,9 @@ guide_gengrob <- function(guide, theme) UseMethod("guide_gengrob")
 matched_aes <- function(layer, guide) {
   all <- names(c(layer$computed_mapping, layer$stat$default_aes))
   geom <- c(layer$geom$required_aes, names(layer$geom$default_aes))
+
+  # Make sure that size guides are shown if a renaming layer is used
+  if (layer$geom$rename_size && "size" %in% all && !"linewidth" %in% all) geom <- c(geom, "size")
   matched <- intersect(intersect(all, geom), names(guide$key))
   matched <- setdiff(matched, names(layer$computed_geom_params))
   setdiff(matched, names(layer$aes_params))
@@ -386,7 +392,7 @@ matched_aes <- function(layer, guide) {
 # `matched` is the set of aesthetics that match between the layer and the guide
 include_layer_in_guide <- function(layer, matched) {
   if (!is.logical(layer$show.legend)) {
-    warn("`show.legend` must be a logical vector.")
+    cli::cli_warn("{.arg show.legend} must be a logical vector.")
     layer$show.legend <- FALSE # save back to layer so we don't issue this warning more than once
     return(FALSE)
   }
