@@ -66,22 +66,24 @@ guide_train.axis <- function(guide, scale, aesthetic = NULL) {
   aesthetic <- aesthetic %||% scale$aesthetics[1]
   breaks <- scale$get_breaks()
 
-  empty_ticks <- new_data_frame(
-    list(aesthetic = numeric(0), .value = numeric(0), .label = character(0))
+  empty_ticks <- data_frame0(
+    aesthetic = numeric(0),
+    .value = numeric(0),
+    .label = character(0)
   )
   names(empty_ticks) <- c(aesthetic, ".value", ".label")
 
   if (length(intersect(scale$aesthetics, guide$available_aes)) == 0) {
-    warn(glue(
-      "axis guide needs appropriate scales: ",
-      glue_collapse(guide$available_aes, ", ", last = " or ")
+    cli::cli_warn(c(
+       "Axis guide lacks appropriate scales",
+       i = "Use one of {.or {.field {guide$available_aes}}}"
     ))
     guide$key <- empty_ticks
   } else if (length(breaks) == 0) {
     guide$key <- empty_ticks
   } else {
     mapped_breaks <- if (scale$is_discrete()) scale$map(breaks) else breaks
-    ticks <- new_data_frame(setNames(list(mapped_breaks), aesthetic))
+    ticks <- data_frame(mapped_breaks, .name_repair = ~ aesthetic)
     ticks$.value <- breaks
     ticks$.label <- scale$get_labels(breaks)
 
@@ -89,7 +91,7 @@ guide_train.axis <- function(guide, scale, aesthetic = NULL) {
   }
 
   guide$name <- paste0(guide$name, "_", aesthetic)
-  guide$hash <- digest::digest(list(guide$title, guide$key$.value, guide$key$.label, guide$name))
+  guide$hash <- hash(list(guide$title, guide$key$.value, guide$key$.label, guide$name))
   guide
 }
 
@@ -120,7 +122,10 @@ guide_transform.axis <- function(guide, coord, panel_params) {
 #' @export
 guide_merge.axis <- function(guide, new_guide) {
   if (!inherits(new_guide, "guide_none")) {
-    warn("guide_axis(): Discarding guide on merge. Do you have more than one guide with the same position?")
+    cli::cli_warn(c(
+      "{.fn guide_axis}: Discarding guide on merge",
+      "i" = "Do you have more than one guide with the same position?"
+    ))
   }
 
   guide
@@ -167,8 +172,7 @@ guide_gengrob.axis <- function(guide, theme) {
 #'
 draw_axis <- function(break_positions, break_labels, axis_position, theme,
                       check.overlap = FALSE, angle = NULL, n.dodge = 1) {
-
-  axis_position <- match.arg(axis_position, c("top", "bottom", "right", "left"))
+  axis_position <- arg_match0(axis_position, c("top", "bottom", "right", "left"))
   aesthetic <- if (axis_position %in% c("top", "bottom")) "x" else "y"
 
   # resolve elements
@@ -246,7 +250,7 @@ draw_axis <- function(break_positions, break_labels, axis_position, theme,
   # break_labels can be a list() of language objects
   if (is.list(break_labels)) {
     if (any(vapply(break_labels, is.language, logical(1)))) {
-      break_labels <- do.call(expression, break_labels)
+      break_labels <- inject(expression(!!!break_labels))
     } else {
       break_labels <- unlist(break_labels)
     }
@@ -278,7 +282,8 @@ draw_axis <- function(break_positions, break_labels, axis_position, theme,
 
   # create gtable
   non_position_sizes <- paste0(non_position_size, "s")
-  label_dims <- do.call(unit.c, lapply(label_grobs, measure_labels_non_pos))
+  label_dims <- lapply(label_grobs, measure_labels_non_pos)
+  label_dims <- inject(unit.c(!!!label_dims))
   grobs <- c(list(ticks_grob), label_grobs)
   grob_dims <- unit.c(max(tick_length, unit(0, "pt")), label_dims)
 
@@ -382,7 +387,7 @@ axis_label_element_overrides <- function(axis_position, angle = NULL) {
 
   # it is not worth the effort to align upside-down labels properly
   if (angle > 90 || angle < -90) {
-    abort("`angle` must be between 90 and -90")
+    cli::cli_abort("{.arg angle} must be between 90 and -90")
   }
 
   if (axis_position == "bottom") {
@@ -410,7 +415,10 @@ axis_label_element_overrides <- function(axis_position, angle = NULL) {
       vjust = if (angle > 0) 1 else if (angle < 0) 0 else 0.5,
     )
   } else {
-    abort(glue("Unrecognized position: '{axis_position}'"))
+    cli::cli_abort(c(
+      "Unrecognized {.arg axis_position}: {.val {axis_position}}",
+      "i" = "Use one of {.val top}, {.val bottom}, {.val left} or {.val right}"
+    ))
   }
 }
 
@@ -432,7 +440,10 @@ warn_for_guide_position <- function(guide) {
     return()
   }
 
-  if (length(unique(guide$key[[position_aes]][breaks_are_unique])) == 1) {
-    warn("Position guide is perpendicular to the intended axis. Did you mean to specify a different guide `position`?")
+  if (length(unique0(guide$key[[position_aes]][breaks_are_unique])) == 1) {
+    cli::cli_warn(c(
+      "Position guide is perpendicular to the intended axis",
+      "i" = "Did you mean to specify a different guide {.arg position}?"
+    ))
   }
 }
