@@ -16,7 +16,7 @@ collide_setup <- function(data, width = NULL, name, strategy,
     }
 
     # Width determined from data, must be floating point constant
-    widths <- unique(data$xmax - data$xmin)
+    widths <- unique0(data$xmax - data$xmin)
     widths <- widths[!is.na(widths)]
 
 #   # Suppress warning message since it's not reliable
@@ -45,11 +45,11 @@ collide <- function(data, width = NULL, name, strategy,
   data <- data[ord, ]
 
   # Check for overlap
-  intervals <- as.numeric(t(unique(data[c("xmin", "xmax")])))
+  intervals <- as.numeric(t(unique0(data[c("xmin", "xmax")])))
   intervals <- intervals[!is.na(intervals)]
 
-  if (length(unique(intervals)) > 1 & any(diff(scale(intervals)) < -1e-6)) {
-    warn(glue("{name} requires non-overlapping x intervals"))
+  if (length(unique0(intervals)) > 1 & any(diff(scale(intervals)) < -1e-6)) {
+    cli::cli_warn("{.fn {name}} requires non-overlapping {.field x} intervals")
     # This is where the algorithm from [L. Wilkinson. Dot plots.
     # The American Statistician, 1999.] should be used
   }
@@ -61,7 +61,7 @@ collide <- function(data, width = NULL, name, strategy,
     data <- dapply(data, "xmin", strategy, ..., width = width)
     data$y <- data$ymax
   } else {
-    abort("Neither y nor ymax defined")
+    cli::cli_abort("Neither {.field y} nor {.field ymax} defined")
   }
   data[match(seq_along(ord), ord), ]
 }
