@@ -6,20 +6,17 @@ StatBindot <- ggproto("StatBindot", Stat,
   required_aes = "x",
   non_missing_aes = "weight",
   default_aes = aes(y = after_stat(count)),
+  dropped_aes = c("bin", "bincenter"), # these are temporary variables that are created and then removed by the stat
 
   setup_params = function(data, params) {
     if (is.null(params$binwidth)) {
-      message("`stat_bindot()` using `bins = 30`. Pick better value with `binwidth`.")
+      cli::cli_inform("Bin width defaults to 1/30 of the range of the data. Pick better value with {.arg binwidth}.")
     }
     params
   },
 
   compute_layer = function(self, data, params, layout) {
-    data <- remove_missing(data, params$na.rm,
-      params$binaxis,
-      snake_class(self),
-      finite = TRUE
-    )
+    data <- remove_missing(data, params$na.rm, name = snake_class(self))
     ggproto_parent(Stat, self)$compute_layer(data, params, layout)
   },
 
@@ -64,16 +61,9 @@ StatBindot <- ggproto("StatBindot", Stat,
                            method = "dotdensity", binpositions = "bygroup",
                            origin = NULL, width = 0.9, drop = FALSE,
                            right = TRUE) {
-
-    # This function taken from integer help page
-    is.wholenumber <- function(x, tol = .Machine$double.eps ^ 0.5) {
-      abs(x - round(x)) < tol
-    }
-
     # Check that weights are whole numbers (for dots, weights must be whole)
-    if (!is.null(data$weight) && any(!is.wholenumber(data$weight)) &&
-        any(data$weight < 0)) {
-      abort("Weights for stat_bindot must be nonnegative integers.")
+    if (!is.null(data$weight) && !(is_integerish(data$weight) && all(data$weight >= 0))) {
+      cli::cli_abort("Weights must be nonnegative integers.")
     }
 
     if (binaxis == "x") {
@@ -110,10 +100,11 @@ StatBindot <- ggproto("StatBindot", Stat,
 
       # Collapse each bin and get a count
       data <- dapply(data, "bincenter", function(x) {
-        new_data_frame(list(
+        data_frame0(
           binwidth = .subset2(x, "binwidth")[1],
-          count = sum(.subset2(x, "weight"))
-        ))
+          count = sum(.subset2(x, "weight")),
+          .size = 1
+        )
       })
 
       if (sum(data$count, na.rm = TRUE) != 0) {
@@ -141,7 +132,7 @@ StatBindot <- ggproto("StatBindot", Stat,
 # It returns a data frame with the original data (x), weights, bin #, and the bin centers.
 densitybin <- function(x, weight = NULL, binwidth = NULL, method = method, range = NULL) {
 
-    if (length(stats::na.omit(x)) == 0) return(new_data_frame())
+    if (length(stats::na.omit(x)) == 0) return(data_frame0())
     if (is.null(weight))  weight <- rep(1, length(x))
     weight[is.na(weight)] <- 0
 
@@ -167,16 +158,17 @@ densitybin <- function(x, weight = NULL, binwidth = NULL, method = method, range
         bin[i] <- cbin
     }
 
-    results <- new_data_frame(list(
+    results <- data_frame0(
       x = x,
       bin = bin,
       binwidth = binwidth,
-      weight = weight
-    ), n = length(x))
+      weight = weight,
+      .size = length(x)
+    )
     results <- dapply(results, "bin", function(df) {
-                    df$bincenter = (min(df$x) + max(df$x)) / 2
-                    return(df)
-                  })
+      df$bincenter = (min(df$x) + max(df$x)) / 2
+      return(df)
+    })
 
     return(results)
 }
