@@ -117,12 +117,12 @@ theme_grey <- function(base_size = 11, base_family = "",
     # Elements in this first block aren't used directly, but are inherited
     # by others
     line =               element_line(
-                           colour = "black", size = base_line_size,
+                           colour = "black", linewidth = base_line_size,
                            linetype = 1, lineend = "butt"
                          ),
     rect =               element_rect(
                            fill = "white", colour = "black",
-                           size = base_rect_size, linetype = 1
+                           linewidth = base_rect_size, linetype = 1
                          ),
     text =               element_text(
                             family = base_family, face = "plain",
@@ -190,13 +190,14 @@ theme_grey <- function(base_size = 11, base_family = "",
     panel.background =   element_rect(fill = "grey92", colour = NA),
     panel.border =       element_blank(),
     panel.grid =         element_line(colour = "white"),
-    panel.grid.minor =   element_line(size = rel(0.5)),
+    panel.grid.minor =   element_line(linewidth = rel(0.5)),
     panel.spacing =      unit(half_line, "pt"),
     panel.spacing.x =    NULL,
     panel.spacing.y =    NULL,
     panel.ontop    =     FALSE,
 
     strip.background =   element_rect(fill = "grey85", colour = NA),
+    strip.clip =         "inherit",
     strip.text =         element_text(
                            colour = "grey10",
                            size = rel(0.8),
@@ -263,7 +264,7 @@ theme_bw <- function(base_size = 11, base_family = "",
       panel.border     = element_rect(fill = NA, colour = "grey20"),
       # make gridlines dark, same contrast with white as in theme_grey
       panel.grid = element_line(colour = "grey92"),
-      panel.grid.minor = element_line(size = rel(0.5)),
+      panel.grid.minor = element_line(linewidth = rel(0.5)),
       # contour strips to match panel contour
       strip.background = element_rect(fill = "grey85", colour = "grey20"),
       # match legend key to background
@@ -291,15 +292,15 @@ theme_linedraw <- function(base_size = 11, base_family = "",
     theme(
       # black text and ticks on the axes
       axis.text        = element_text(colour = "black", size = rel(0.8)),
-      axis.ticks       = element_line(colour = "black", size = rel(0.5)),
+      axis.ticks       = element_line(colour = "black", linewidth = rel(0.5)),
       # NB: match the *visual* thickness of axis ticks to the panel border
       #     0.5 clipped looks like 0.25
 
       # pure black panel border and grid lines, but thinner
-      panel.border     = element_rect(fill = NA, colour = "black", size = rel(1)),
+      panel.border     = element_rect(fill = NA, colour = "black", linewidth = rel(1)),
       panel.grid       = element_line(colour = "black"),
-      panel.grid.major = element_line(size = rel(0.1)),
-      panel.grid.minor = element_line(size = rel(0.05)),
+      panel.grid.major = element_line(linewidth = rel(0.1)),
+      panel.grid.minor = element_line(linewidth = rel(0.05)),
 
       # strips with black background and white text
       strip.background = element_rect(fill = "black"),
@@ -330,15 +331,15 @@ theme_light <- function(base_size = 11, base_family = "",
     theme(
       # white panel with light grey border
       panel.background = element_rect(fill = "white", colour = NA),
-      panel.border     = element_rect(fill = NA, colour = "grey70", size = rel(1)),
+      panel.border     = element_rect(fill = NA, colour = "grey70", linewidth = rel(1)),
       # light grey, thinner gridlines
       # => make them slightly darker to keep acceptable contrast
       panel.grid       = element_line(colour = "grey87"),
-      panel.grid.major = element_line(size = rel(0.5)),
-      panel.grid.minor = element_line(size = rel(0.25)),
+      panel.grid.major = element_line(linewidth = rel(0.5)),
+      panel.grid.minor = element_line(linewidth = rel(0.25)),
 
       # match axes ticks thickness to gridlines and colour to panel border
-      axis.ticks       = element_line(colour = "grey70", size = rel(0.5)),
+      axis.ticks       = element_line(colour = "grey70", linewidth = rel(0.5)),
 
       # match legend key to panel.background
       legend.key       = element_rect(fill = "white", colour = NA),
@@ -376,11 +377,11 @@ theme_dark <- function(base_size = 11, base_family = "",
       # inverse grid lines contrast compared to theme_grey
       # make them thinner and try to keep the same visual contrast as in theme_light
       panel.grid       = element_line(colour = "grey42"),
-      panel.grid.major = element_line(size = rel(0.5)),
-      panel.grid.minor = element_line(size = rel(0.25)),
+      panel.grid.major = element_line(linewidth = rel(0.5)),
+      panel.grid.minor = element_line(linewidth = rel(0.25)),
 
       # match axes ticks thickness to gridlines
-      axis.ticks       = element_line(colour = "grey20", size = rel(0.5)),
+      axis.ticks       = element_line(colour = "grey20", linewidth = rel(0.5)),
 
       # match legend key to panel.background
       legend.key       = element_rect(fill = "grey50", colour = NA),
@@ -440,13 +441,13 @@ theme_classic <- function(base_size = 11, base_family = "",
       panel.grid.minor = element_blank(),
 
       # show axes
-      axis.line      = element_line(colour = "black", size = rel(1)),
+      axis.line      = element_line(colour = "black", linewidth = rel(1)),
 
       # match legend key to panel.background
       legend.key       = element_blank(),
 
       # simple, black and white strips
-      strip.background = element_rect(fill = "white", colour = "black", size = rel(2)),
+      strip.background = element_rect(fill = "white", colour = "black", linewidth = rel(2)),
       # NB: size is 1 but clipped, it looks like the 0.5 of the axes
 
       complete = TRUE
@@ -484,6 +485,7 @@ theme_void <- function(base_size = 11, base_family = "",
     legend.position =    "right",
     legend.text =        element_text(size = rel(0.8)),
     legend.title =       element_text(hjust = 0),
+    strip.clip =         "inherit",
     strip.text =         element_text(size = rel(0.8)),
     strip.switch.pad.grid = unit(half_line / 2, "pt"),
     strip.switch.pad.wrap = unit(half_line / 2, "pt"),
@@ -529,12 +531,12 @@ theme_test <- function(base_size = 11, base_family = "",
 
   t <- theme(
     line =               element_line(
-                           colour = "black", size = base_line_size,
+                           colour = "black", linewidth = base_line_size,
                            linetype = 1, lineend = "butt"
                          ),
     rect =               element_rect(
                            fill = "white", colour = "black",
-                           size = base_rect_size, linetype = 1
+                           linewidth = base_rect_size, linetype = 1
                          ),
     text =               element_text(
                             family = base_family, face = "plain",
@@ -609,6 +611,7 @@ theme_test <- function(base_size = 11, base_family = "",
     panel.ontop    =     FALSE,
 
     strip.background =   element_rect(fill = "grey85", colour = "grey20"),
+    strip.clip =         "inherit",
     strip.text =         element_text(
                            colour = "grey10",
                            size = rel(0.8),
@@ -616,6 +619,7 @@ theme_test <- function(base_size = 11, base_family = "",
                          ),
     strip.text.x =       NULL,
     strip.text.y =       element_text(angle = -90),
+    strip.text.y.left =  element_text(angle = 90),
     strip.placement =    "inside",
     strip.placement.x =  NULL,
     strip.placement.y =  NULL,
@@ -666,5 +670,5 @@ theme_all_null <- function() {
   )
 
   args <- c(elements, list(complete = TRUE))
-  do.call(theme, args)
+  inject(theme(!!!args))
 }
