@@ -1,5 +1,3 @@
-context("scale_gradient")
-
 # Limits ------------------------------------------------------------------
 
 test_that("points outside the limits are plotted as NA", {
@@ -9,5 +7,5 @@ test_that("points outside the limits are plotted as NA", {
     scale_fill_gradient2(limits = c(-1, 1), midpoint = 2, na.value = "orange")
 
   correct_fill <- c("#B26D65", "#DCB4AF", "orange")
-  expect_equivalent(layer_data(p)$fill, correct_fill)
+  expect_equal(layer_data(p)$fill, correct_fill)
 })
